package org.xbib.interlibrary.client.gvi;

import org.xbib.interlibrary.action.avail.AvailService;
import org.xbib.interlibrary.action.avail.AvailServiceProvider;
import org.xbib.interlibrary.action.resolve.ResolveService;
import org.xbib.interlibrary.action.resolve.ResolveServiceProvider;
import org.xbib.interlibrary.action.search.SearchService;
import org.xbib.interlibrary.action.search.SearchServiceProvider;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.client.gvi.sru.SRUAvailService;
import org.xbib.interlibrary.client.gvi.sru.SRUResolveService;
import org.xbib.interlibrary.client.gvi.sru.SRUSearchService;
import org.xbib.interlibrary.client.gvi.z.ZAvailService;
import org.xbib.interlibrary.client.gvi.z.ZResolveService;
import org.xbib.interlibrary.client.gvi.z.ZSearchService;
import org.xbib.interlibrary.sru.SRUClientHelper;

public class Provider {

    volatile static SRUClientHelper clientHelper;

    public Provider() {
    }

    public static class SRUResolveServiceProvider implements ResolveServiceProvider {

        private SRUResolveService sruResolveService;

        public SRUResolveServiceProvider() {
        }

        @Override
        public ResolveService createService(ServiceArguments arguments) {
            if (clientHelper == null) {
                clientHelper = new SRUClientHelper(arguments.getSettings().getAsSettings("sru." + arguments.getName()));
            }
            if (sruResolveService == null) {
                sruResolveService = new SRUResolveService(arguments, clientHelper);
            }
            return sruResolveService;
        }
    }

    public static class SRUAvailServiceProvider implements AvailServiceProvider {

        private SRUAvailService sruAvailService;

        public SRUAvailServiceProvider() {
        }

        @Override
        public AvailService createService(ServiceArguments arguments) {
            if (clientHelper == null) {
                clientHelper = new SRUClientHelper(arguments.getSettings().getAsSettings("sru." + arguments.getName()));
            }
            if (sruAvailService == null) {
                sruAvailService = new SRUAvailService(arguments, clientHelper);
            }
            return sruAvailService;
        }
    }

    public static class SRUSearchServiceProvider implements SearchServiceProvider {

        private SRUSearchService sruSearchService;

        public SRUSearchServiceProvider() {
        }

        @Override
        public SearchService createService(ServiceArguments arguments) {
            if (clientHelper == null) {
                clientHelper = new SRUClientHelper(arguments.getSettings().getAsSettings("sru." + arguments.getName()));
            }
            if (sruSearchService == null) {
                sruSearchService = new SRUSearchService(arguments, clientHelper);
            }
            return sruSearchService;
        }
    }

    public static class ZAvailServiceProvider implements AvailServiceProvider {

        private ZAvailService ZAvailService;

        public ZAvailServiceProvider() {
        }

        @Override
        public AvailService createService(ServiceArguments arguments) {
            if (ZAvailService == null) {
                ZAvailService = new ZAvailService(arguments);
            }
            return ZAvailService;
        }
    }

    public static class ZResolveServiceProvider implements ResolveServiceProvider {

        private ZResolveService resolveService;

        public ZResolveServiceProvider() {
        }

        @Override
        public ResolveService createService(ServiceArguments arguments) {
            if (resolveService == null) {
                resolveService = new ZResolveService(arguments);
            }
            return resolveService;
        }
    }

    public static class ZSearchServiceProvider implements SearchServiceProvider {

        private ZSearchService searchService;

        public ZSearchServiceProvider() {
        }

        @Override
        public SearchService createService(ServiceArguments arguments) {
            if (searchService == null) {
                searchService = new ZSearchService(arguments);
            }
            return searchService;
        }
    }

}
