package org.xbib.interlibrary.client.gvi.sru;

import java.util.Locale;
import java.util.function.Consumer;
import org.xbib.datastructures.common.MultiMap;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.action.avail.AvailRequest;
import org.xbib.interlibrary.action.avail.AvailResponse;
import org.xbib.interlibrary.action.avail.AvailResponseBuilder;
import org.xbib.interlibrary.action.avail.AvailServiceResponseBuilder;
import org.xbib.interlibrary.api.library.LibraryService;
import org.xbib.interlibrary.api.action.Service;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.sru.AbstractSRUAvailService;
import org.xbib.interlibrary.sru.DisplayHelper;
import org.xbib.interlibrary.sru.SRUClientHelper;
import org.xbib.marc.MarcRecord;
import org.xbib.marc.MarcRecordListener;

public class SRUAvailService extends AbstractSRUAvailService {

    public SRUAvailService(ServiceArguments arguments, SRUClientHelper sruClientHelper) {
        super(arguments, sruClientHelper);
    }

    @Override
    protected String supports(String source) {
        return source != null && source.toLowerCase(Locale.ROOT).contains("gvi") ? "GVI" : null;
    }

    @Override
    protected MarcRecordListener getMarcRecordListener(AvailRequest availRequest,
                                                       AvailServiceResponseBuilder availResponse,
                                                       MultiMap<String, LibraryService> multiMap) {
        return new MarcRecordListener() {

            @Override
            public void beginCollection() {
            }

            @Override
            public void record(MarcRecord marcRecord) {
                if (availRequest.isBibliographicDescriptionEnabled()) {
                    availResponse.setBibliographicDescription(marcRecord);
                    availResponse.setDisplay(DisplayHelper.display(marcHelper, marcRecord, getName(), getName().toUpperCase(Locale.ROOT), null,
                            marcHelper.getRecordIdSupplier(marcRecord)));
                }
                for (LibraryService service : extractGermanInterlibraryServices(availRequest, marcRecord)) {
                    multiMap.put(service.getISIL(), service);
                }
            }

            @Override
            public void endCollection() {
            }
        };
    }

    @Override
    public int compareTo(Service<AvailRequest, AvailResponse, AvailResponseBuilder> o) {
        return getName().compareTo(o.getName());
    }

    protected String normalizeSourceId(String sourceId) {
        return sourceId.trim();
    }

    @Override
    protected String sourceIdCQL(String sourceId) {
        return "rec.id = \"" + sourceId + "\"";
    }

    @Override
    protected Consumer<TinyMap.Builder<String, Object>> parameterCompletion() {
        // startRecord not supported
        return params -> {
            params.put("maximumRecords", 1);
            params.put("stylesheet", "");
        };
    }
}
