package org.xbib.interlibrary.client.gvi.z;

import org.xbib.datastructures.common.MultiMap;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.action.avail.AvailRequest;
import org.xbib.interlibrary.action.avail.AvailResponse;
import org.xbib.interlibrary.action.avail.AvailResponseBuilder;
import org.xbib.interlibrary.action.avail.AvailServiceResponseBuilder;
import org.xbib.interlibrary.api.library.Library;
import org.xbib.interlibrary.api.library.LibraryService;
import org.xbib.interlibrary.api.action.Response;
import org.xbib.interlibrary.api.library.DefaultLibraryService;
import org.xbib.interlibrary.api.action.Service;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.api.library.Material;
import org.xbib.interlibrary.api.library.Mode;
import org.xbib.interlibrary.z.AbstractZAvailService;
import org.xbib.interlibrary.z.DisplayHelper;
import org.xbib.marc.MarcField;
import org.xbib.marc.MarcRecord;

import java.util.List;
import java.util.Locale;
import static org.xbib.interlibrary.api.library.Material.ONLINE_RESOURCE;

public class ZAvailService extends AbstractZAvailService {

    public ZAvailService(ServiceArguments arguments) {
        super(arguments);
    }

    @Override
    protected String supports(String source) {
        return source.toLowerCase(Locale.ROOT).contains("kobv") ? "KOBV" : null;
    }

    @Override
    protected void avail(AvailRequest availRequest,
                         AvailServiceResponseBuilder availResponse,
                         MultiMap<String, LibraryService> multiMap,
                         MarcRecord marcRecord) {
        boolean found = false;
        if (availRequest.isBibliographicDescriptionEnabled()) {
            availResponse.setBibliographicDescription(marcRecord);
            TinyMap.Builder<String, Object> builder = TinyMap.builder();
            DisplayHelper.display(builder, marcHelper, marcRecord, getName(), getName().toUpperCase(Locale.ROOT), null,
                    marcHelper.getRecordIdSupplier(marcRecord));
            availResponse.setDisplay(builder.build());
        }
        String sourceid = identifier(marcRecord);
        List<Material> materials = marcHelper.extractMaterial(marcRecord);
        for (MarcField marcField : marcRecord.getAll("049")) {
            DefaultLibraryService.Builder serviceBuilder = DefaultLibraryService.builder();
            String value = marcField.getFirstSubfieldValue("a");
            int pos = value.indexOf(';');
            String isil = pos > 0 ? value.substring(0, pos) : value;
            //String id = pos > 0 ? value.substring(pos + 1) : null;
            if (!isil.isEmpty()) {
                Library library = configuration.getLibrary(isil);
                if (library != null) {
                    serviceBuilder.setISIL(library.getISIL());
                    serviceBuilder.setName(library.getName());
                    serviceBuilder.setDomainPriority(domainPriority(availRequest, library));
                    serviceBuilder.setLibraryPriority(libraryPriority(availRequest, library));
                    serviceBuilder.setMaterials(materials);
                    serviceBuilder.setServicePriority(materials.contains(ONLINE_RESOURCE) ? 1 : 2);
                    serviceBuilder.setMode(Mode.LOAN_COPY);
                    if (library.getDomain() != null) {
                        serviceBuilder.setSource(library.getDomain().getOrganization());
                    }
                    serviceBuilder.setSourceId(sourceid);
                    serviceBuilder.setId(Integer.toString(random()));
                }
                multiMap.put(isil, serviceBuilder.build());
                found = true;
            }
        }
        if (found) {
            availResponse.setStatus(getName(), Response.Status.OK);
        } else {
            availResponse.setStatus(getName(), Response.Status.EMPTY_RESPONSE);
        }
    }

    @Override
    protected String normalizeSourceId(String sourceId) {
        return sourceId;
    }

    @Override
    public int compareTo(Service<AvailRequest, AvailResponse, AvailResponseBuilder> o) {
        return getName().compareTo(o.getName());
    }
}
